@@ -7,7 +7,8 @@ from .status import status as sts
 """
 TODOS:
 - Send request params to app.
-- Add dynamic content-length and date heaaders to response.
+- Add dynamic content-length and date headers to response.
+- Make query params accessible from app.
 """
 
 # defaults
@@ -20,8 +21,9 @@ class Request:
     def __init__(self, parsed_req):
         self.__parsed_req = parsed_req
         self.method = parsed_req["method"]
-        self.url = parsed_req["url"]
+        self.path = parsed_req["path"]
         self.body = parsed_req["body"]
+        self.params = parsed_req["params"]
         
         
     def get_data(self, data):
@@ -32,7 +34,11 @@ class Response:
     def __init__(self):
         self.__status = 200
         self.__status_message = sts[str(self.__status)]
-        self.__headers = { "content-type": "text/html; charset=utf-8", "connection": "close", "server": "Python/Tea" }
+        self.__headers = {
+            "content-type": "text/html; charset=utf-8",
+            "connection": "close",
+            "server": "Python/Tea"
+        }
         self.__body = ""
     
     
@@ -75,7 +81,6 @@ class Tea:
         parsed_path = parse_path(path)
         self.__rules.append({ "method": "GET", "path": parsed_path, "callback": callback })
         
-        
     def post(self, path, callback):
         parsed_path = parse_path(path)
         self.__rules.append({ "method": "POST", "path": parsed_path, "callback": callback })
@@ -85,31 +90,37 @@ class Tea:
         parsed_req = parse_raw_http_req(req)
         
         if self.__mode == "development":
-            print(f"[{datetime.now().strftime('%H.%M.%S')}] > {parsed_req['method']} http://{self.__host}:{self.__port}{parsed_req['url']}")
+            print(f"[{datetime.now().strftime('%H.%M.%S')}] > {parsed_req['method']} http://{self.__host}:{self.__port}{parsed_req['path']['full_path']}")
         
         # default error message
         res_text = f"{HTTP_VERSION} 404 NOT FOUND\r\n\r\n404 NOT FOUND\r\n"
         
-        # we have to know method count on specific path to check whenever send 405 error
-        method_count_on_specific_path = len(list(filter(lambda x: x["path"]["full_path"] == parsed_req["url"], self.__rules)))
-        
-        for rule in self.__rules:
-            if parsed_req["url"] == rule["path"]["full_path"] and parsed_req["method"] == rule["method"]:
+        # I know the code below is hard to read and understand but just wait for the comment lines. I will add soon.
+        same_route_count = list(filter(lambda r: r["path"]["route_count"] == parsed_req["path"]["route_count"], self.__rules))
+        if same_route_count:
+            res_text = f"{HTTP_VERSION} 405 METHOD NOT ALLOWED\r\n\r\n405 METHOD NOT ALLOWED\r\n"
+            same_method = list(filter(lambda r: r["method"] == parsed_req["method"], same_route_count))
+            if same_method:
+                same_default_path = list(filter(lambda r: r["path"]["full_path"] == parsed_req["path"]["full_path"], same_method))
+                current_rule = {}
+                if not same_default_path:
+                    if len(same_method) > 1:
+                        current_rule = list(filter(lambda r: r["path"]["full_path"] != parsed_req["path"]["full_path"], same_method))[0]
+                    else:
+                        current_rule = same_method[0]
+                    for prompted_param in current_rule["path"]["prompted_params"]: # (param_name, param_index_in_path)
+                        parsed_req["params"][prompted_param[0]] = parsed_req["path"]["routes"][prompted_param[1]]
+                else:
+                    current_rule = same_default_path[0]
+                
                 req = Request(parsed_req)
                 res = Response()
                 
                 # run callback function to send request to app and get response
-                rule["callback"](req, res)
+                current_rule["callback"](req, res)
                 
                 # get response text from app
                 res_text = res.get_full_res_text().replace("'", '"')
-                break
-            elif method_count_on_specific_path > 1:
-                method_count_on_specific_path -= 1
-                continue
-            else:
-                res_text = f"{HTTP_VERSION} 405 METHOD NOT ALLOWED\r\n\r\n405 METHOD NOT ALLOWED\r\n"
-        
         # send response text
         conn.sendall(res_text.encode())
      
